/*
	Mod support info:
	One can pretty much simply copy the "if not UniqueSuper" block below this info and the "do" block below that and just change some stuff.
	However, that's a bit underwhelming of mod support info so I guess I'll add annotations to those blocks.
*/
if not UniqueSuper then //This allows this mod and thy mod to be loaded in either order without errors. It creates the basic structure of the UniqueSuper table if it does not already exist.
	rawset(_G, "UniqueSuper", {
	superTable = {},
	specialThemes = {}})
end

do
	local addToSuperTable = {
	knuckles = {"vsknux", "s3ksup"},
	sonic = {"ssonic", "ssonsc", "s3ksup"},
	tails = {"s3ksup"},
	metalsonic = {"sdswbf", "noelsa"},
	vector = {"chaotx"},
	espio = {"chaotx"},
	charmy = {"chaotx"},
	flicky = {"pupaa1"},
	vexermighty = {"manias"},
	mighty = {"manias"},
	ray = {"manias"},
	silver = {"slaris"},
	mechasonic = {"doomsd"},
	eggman = {"bigarm"},
	metalknuckles = {"vmknux"},
	deton = {"metmad"},
	shadow = {"livlrn"},
	blaze = {"wrapbl"},
	snolf = {"ssonic", "ssonsc", "s3ksup"}}
	for skin, themes in pairs(addToSuperTable) do //Go through the character super theme list defined in addToSuperTable
		if not UniqueSuper.superTable[skin] then //If the specified skin does not already have a super theme list, give it an empty list.
			UniqueSuper.superTable[skin] = {}
		end
		for _, theme in ipairs(themes) do //Go through the themes in the list of super themes to add and add them to the character's theme list.
			table.insert(UniqueSuper.superTable[skin], theme)
		end
	end
	
	local addToSpecialThemes = {"VSALT", "VSBOSS", "VSFANG", "BCZ1", "VSMETL", "VSBRAK", "MP_MRZ"} //The "special themes" are themes that should not be replaced with super themes.
	for _, theme in ipairs(addToSpecialThemes) do
		table.insert(UniqueSuper.specialThemes, theme)
	end
end

addHook("ThinkFrame", do
	for player in players.iterate
		if (player.mo and player.mo.valid)
			if not UniqueSuper.superTable[player.mo.skin]
				player.mo.rtheme = "_super"
				player.mo.rthemeskin = player.mo.skin
			end
			if (not player.mo.rtheme) or ((player.mo.rthemeskin) and (player.mo.rthemeskin ~= player.mo.skin)) then
				player.mo.rtheme = UniqueSuper.superTable[player.mo.skin][P_RandomRange(1, #(UniqueSuper.superTable[player.mo.skin]))]
				player.mo.rthemeskin = player.mo.skin
			end
			if mapmusname == S_MusicName() then
				player.normMusPosition = S_GetMusicPosition()
			end
			if player.powers[pw_super] then
				local theme = player.stheme or player.mo.rtheme or "_super"
				if ((not player.powers[pw_extralife]) and (not player.exiting)) and ((mapmusname == theme) or (not UniqueSuper.specialThemes[mapmusname]))
					S_ChangeMusic(theme,true,player,0,player.supMusPosition)
					player.supMusPosition = S_GetMusicPosition()
				elseif (UniqueSuper.specialThemes[mapmusname])
					S_ChangeMusic(mapmusname,true,player,0,player.normMusPosition)
				end
			else
				player.supMusPosition = 0
			end
		end
	end
end)

COM_AddCommand("setsupertheme", function(player, ...)
	local arg = {...}
	local skin, variant, track
	if #arg == 0 then
		player.stheme = nil
		CONS_Printf(player, "Setting super theme to default.\nFunction usage:\nsetsupertheme\nsetsupertheme [skin]\nsetsupertheme [variant]\nsetsupertheme [skin] [variant]\nsetsupertheme default\nIf called without any parameters, sets super theme to skin default and displays this.\nIf called with skin parameter, sets super theme to that skin's variant 1; entering a variant parameter sets super theme to that skin's variant theme.\nIf called with the parameter 'default', sets super theme to SRB2 default.")
		return
	elseif #arg == 1 then
		if arg[1] == "default" then
			player.stheme = "_super"
			CONS_Printf(player, "Setting super theme to SRB2 default.")
			return
		elseif UniqueSuper.superTable[arg[1]] then
			player.stheme = UniqueSuper.superTable[arg[1]][1]
			CONS_Printf(player, "Setting super theme to \"" + player.stheme + "\"; variant 1 of " + #(UniqueSuper.superTable[arg[1]]) + " of the super theme of " + arg[1] + ".")
			return
		elseif tonumber(arg[1]) then
			if not UniqueSuper.superTable[player.realmo.skin] then
				CONS_Printf(player, "Error: No unique super themes exist for " + player.realmo.skin + ". Setting super theme to default.")
			end
			player.stheme = UniqueSuper.superTable[player.realmo.skin][tonumber(arg[1])]
			if not player.stheme then
				CONS_Printf(player, "Error: " + player.realmo.skin + " has only " + #(UniqueSuper.superTable[player.realmo.skin]) + " super theme variants; setting theme to variant 1.")
				player.stheme = UniqueSuper.superTable[player.realmo.skin][1]
				return
			end
			CONS_Printf(player, "Setting super theme to \"" + player.stheme + "\"; variant " + arg[1] + " of " + #(UniqueSuper.superTable[player.realmo.skin]) + " of the super theme of " + player.realmo.skin + ".")
			return
		else
			player.stheme = nil
			CONS_Printf(player, "Error: Input was neither a skin there exist super themes for, a number or default. Setting super theme to default.")
			return
		end
	elseif #arg == 2 then
		if not UniqueSuper.superTable[arg[1]] then
			player.sthene = nil
			CONS_Printf(player, "Error: No unique super themes exist for " + arg[1] + ". Setting super theme to default.")
			return
		elseif not UniqueSuper.superTable[arg[1]][tonumber(arg[2])] then
			player.stheme = UniqueSuper.superTable[arg[1]][1]
			CONS_Printf(player, "Error: " + arg[1] + " has only " + #(UniqueSuper.superTable[arg[1]]) + " super theme variants; setting theme to variant 1.")
			return
		else
			player.stheme = UniqueSuper.superTable[arg[1]][tonumber(arg[2])]
			CONS_Printf(player, "Setting super theme to \"" + player.stheme + "\"; variant " + arg[2] + " of " + #(UniqueSuper.superTable[arg[1]]) + " of the super theme of " + arg[1] + ".")
			return
		end
	end
	CONS_Printf(player, "Error: Too many inputs.")
end)